#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from . import mpfun
from models import *
from sqlalchemy.orm import Session
from database import get_local_db

# 建筑工程信息表

# 加密和HMAC签名
def sign_row(db: Session, row: BuildingProjectInfo) -> None:
    if row.sign != '':
        return
    
    prjcode = mpfun.base64_data(row.prjcode) # 项目编号
    prjname = mpfun.base64_data(row.prjname) # 项目名称
    letternum = mpfun.base64_data(row.letternum) # 立项文号

    prjtypename = mpfun.base64_data(row.prjtypename) # 项目类型
    issueby = mpfun.base64_data(row.issueby) # 立项批复机关

    address = mpfun.base64_data(row.address) # 项目地址
    prjpropertyname = mpfun.base64_data(row.prjusefor) # 建设性质
    prjusefor = mpfun.base64_data(row.prjusefor) # 工程用途
    ownername = mpfun.base64_data(row.ownername) # 建设单位
    
    sign_data = ",".join([prjcode, prjname, letternum, prjtypename, issueby, address, prjpropertyname, prjusefor, ownername])
    sign_hmac = mpfun.sign_data(sign_data)
    # print('sign_tbl_user sign_data:', sign_data)
    # print('sign_tbl_user sign_hmac:', sign_hmac)

    row.sign = sign_hmac
    
    db.commit()

# 比较字段合并字符串是否和MAC值匹配上,调用密码服务器[验证HMAC]接口
def sign_valid_row(row: BuildingProjectInfo) -> bool:
    if row.sign == '':
        return True

    # 关键字段合并字符串
    sign_data = get_sign_str(row)
    # print('sys_user sign_data:', sign_data)
    
    # 原HMACSM3数值
    sign_hmac = row.sign
    # print('sign_hmac:', sign_hmac)

    return mpfun.hmac_verify(sign_data, sign_hmac)

# 生成待签名的字符串
def get_sign_str(row: BuildingProjectInfo) -> str:
    prjcode = mpfun.base64_data(row.prjcode) # 项目编号
    prjname = mpfun.base64_data(row.prjname) # 项目名称
    letternum = mpfun.base64_data(row.letternum) # 立项文号

    prjtypename = mpfun.base64_data(row.prjtypename) # 项目类型
    issueby = mpfun.base64_data(row.issueby) # 立项批复机关

    address = mpfun.base64_data(row.address) # 项目地址
    prjpropertyname = mpfun.base64_data(row.prjusefor) # 建设性质
    prjusefor = mpfun.base64_data(row.prjusefor) # 工程用途
    ownername = mpfun.base64_data(row.ownername) # 建设单位

    # 关键字段合并字符串
    sign_data = ",".join([prjcode, prjname, letternum, prjtypename, issueby, address, prjpropertyname, prjusefor, ownername])
    return sign_data

# 生成HAMC签名值
def get_sign_hmac(row: BuildingProjectInfo) -> str:
    sign_data = get_sign_str(row)
    return mpfun.sign_data(sign_data) 

# 对所有数据进行签名
def sign_table():
    print('sign_building_project_info table =====>>>')
    with get_local_db() as db:
        rows = db.query(BuildingProjectInfo).filter(BuildingProjectInfo.sign == '').all()
        for row in rows:
            sign_row(db, row)