#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from . import mpfun
from models import *
from utils import *
from sqlalchemy.orm import Session
from database import get_local_db

# 图层菜单表

# 加密和HMAC签名
def sign_row(db: Session, row: SysMenuLayer) -> None:
    if row.sign != '':
        return
    
    menu_id = str(row.menu_id) # 菜单ID
    menu_name = mpfun.base64_data(row.menu_name) # 菜单名称
    parent_id = str(row.parent_id) # 父菜单ID
    order_num = str(row.order_num) # 显示顺序
    path = mpfun.base64_data(row.path) # 路由地址
    component = mpfun.base64_data(row.component) # 组件路径
    menu_type = row.menu_type # 菜单类型
    visible = row.visible # 显示状态
    status = str(row.status) # 状态
    del_flag = row.del_flag # 删除标志

    sign_data = ",".join([menu_id, menu_name, parent_id, order_num, path, component, menu_type, visible, status, del_flag])
    sign_hmac = mpfun.sign_data(sign_data)
    # print('sign_tbl_post sign_data:', sign_data)
    # print('sign_tbl_post sign_hmac:', sign_hmac)

    row.sign = sign_hmac
    
    db.commit()
    
# 比较字段合并字符串是否和MAC值匹配上,调用密码服务器[验证HMAC]接口
def sign_valid_row(row: SysMenuLayer) -> bool:
    if row.sign == '':
        return True

    # 关键字段合并字符串
    sign_data = get_sign_str(row)
    # print('sys_post sign_data:', sign_data)
    
    # 原HMACSM3数值
    sign_hmac = row.sign
    # print('sign_hmac:', sign_hmac)

    return mpfun.hmac_verify(sign_data, sign_hmac)

# 生成待签名的字符串
def get_sign_str(row: SysMenuLayer) -> str:
    menu_id = str(row.menu_id) # 菜单ID
    menu_name = mpfun.base64_data(row.menu_name) # 菜单名称
    parent_id = str(row.parent_id) # 父菜单ID
    order_num = str(row.order_num) # 显示顺序
    path = mpfun.base64_data(row.path) # 路由地址
    component = mpfun.base64_data(row.component) # 组件路径
    menu_type = row.menu_type # 菜单类型
    layer_visible = row.layer_visible # 显示状态
    status = str(row.status) # 状态
    del_flag = row.del_flag # 删除标志

    # 关键字段合并字符串
    sign_data = ",".join([menu_id, menu_name, parent_id, order_num, path, component, menu_type, layer_visible, status, del_flag])
    return sign_data

# 生成HAMC签名值
def get_sign_hmac(row: SysMenuLayer) -> str:
    sign_data = get_sign_str(row)
    return mpfun.sign_data(sign_data)

# 对所有数据进行签名
def sign_table():
    print('sign_sys_menu_layer table =====>>>')
    with get_local_db() as db:
        rows = db.query(SysMenuLayer).filter(SysMenuLayer.sign == '').all()
        for row in rows:
            sign_row(db, row)