#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from . import mpfun
from models import *
from utils import *
from sqlalchemy.orm import Session
from database import get_local_db

# 岗位表

# 加密和HMAC签名
def sign_row(db: Session, row: SysPost) -> None:
    if row.sign != '':
        return
    
    post_id = str(row.post_id) # 岗位ID
    dept_id = str(row.dept_id) # 部门ID
    post_code = row.post_code # 岗位代码
    post_name = mpfun.base64_data(row.post_name) # 岗位名称
    post_sort = str(row.post_sort) # 排序
    status = str(row.status) # 状态

    sign_data = ",".join([post_id, dept_id, post_code, post_name, post_sort, status])
    sign_hmac = mpfun.sign_data(sign_data)
    # print('sign_tbl_post sign_data:', sign_data)
    # print('sign_tbl_post sign_hmac:', sign_hmac)

    row.sign = sign_hmac
    
    db.commit()
    
# 比较字段合并字符串是否和MAC值匹配上,调用密码服务器[验证HMAC]接口
def sign_valid_row(row: SysPost) -> bool:
    if row.sign == '':
        return True

    # 关键字段合并字符串
    sign_data = get_sign_str(row)
    # print('sys_post sign_data:', sign_data)
    
    # 原HMACSM3数值
    sign_hmac = row.sign
    # print('sign_hmac:', sign_hmac)

    return mpfun.hmac_verify(sign_data, sign_hmac)

# 生成待签名的字符串
def get_sign_str(row: SysPost) -> str:
    post_id = str(row.post_id) # 岗位ID
    dept_id = str(row.dept_id) # 部门ID
    post_code = row.post_code # 岗位代码
    post_name = mpfun.base64_data(row.post_name) # 岗位名称
    post_sort = str(row.post_sort) # 排序
    status = str(row.status) # 状态

    # 关键字段合并字符串
    sign_data = ",".join([post_id, dept_id, post_code, post_name, post_sort, status])
    return sign_data

# 生成HAMC签名值
def get_sign_hmac(row: SysPost) -> str:
    sign_data = get_sign_str(row)
    return mpfun.sign_data(sign_data)

# 对所有数据进行签名
def sign_table():
    print('sign_sys_post table =====>>>')
    with get_local_db() as db:
        rows = db.query(SysPost).filter(SysPost.sign == '').all()
        for row in rows:
            sign_row(db, row)