#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from sqlalchemy import String, Column, Integer, Date, DateTime
from database import Base
from datetime import datetime

class YzyOrgUnitEntity(Base):
    """
    组织机构信息表
    """
    __tablename__ = "yzy_org_unit"

    unitid = Column(String, primary_key=True)
    unitname = Column(String)
    createtime = Column(Date)
    updatetime = Column(Date)
    unitpath = Column(String)
    parentunitid = Column(String)
    order = Column(Integer)
    weworkpartyid = Column(String)
    orgtype = Column(String)
    priority = Column(Integer)

    class Config:
        orm_mode = True



class YzyOrgUserEntity(Base):
    """
    用户信息表
    """
    __tablename__ = "yzy_org_user"

    uid = Column(Integer, primary_key=True, autoincrement=True)
    userid = Column(String, primary_key=True)
    username = Column(String)
    displayname = Column(String)
    account = Column(String)
    gender = Column(Integer)
    telephonenumber = Column(String)
    certificatetypeid = Column(Integer)
    certificatenumber = Column(String)
    #weworkpartyid = Column(String)
    createtime = Column(Date)
    updatetime = Column(Date)
    unitid = Column(String)
    #unitleader = Column(Integer)
    position = Column(String)
    priority = Column(Integer)
    unitpath = Column(Integer)


    class Config:
        orm_mode = True


class YzyMsgQueue(Base):
    """
    粤政易消息队列
    """
    __tablename__ = "yzy_msg_queue"

    id = Column(Integer, primary_key=True, autoincrement=True)
    yzy_userid = Column(String, default='', server_default='', comment='粤政易用户ID')
    mobile = Column(String, default='', server_default='', comment='手机号码')
    create_time = Column(DateTime, default=datetime.now, comment='数据创建时间')
    sent_time = Column(String, comment='发送时间')
    sent_status = Column(Integer, default=0, server_default='0', comment='发送状态 0 待发 1发送中 2 发送成功 9 发送失败')
    content = Column(String, default='', server_default='', comment='消息体')
    recorded_by = Column(Integer, default=0, server_default='0', comment='记录用户ID')
    detail_url = Column(String, default='', server_default='', comment='跳转URL')
    title = Column(String, default='', server_default='', comment='标题')
    foreign_key = Column(String, default='', server_default='', comment='文件外键 --技术字段')
    from_scenario = Column(String, default='', server_default='', comment='对应标识 --技术字段')
    errcode = Column(Integer, default=0, server_default='0', comment='错误代码')
    errmsg = Column(String, default='', server_default='', comment='错误提示')
    

class YssYstUploadFileEntity(Base):
    """
    上传文件
    """
    __tablename__ = "yssyst_upload_file"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    sfzh = Column(String, default='')
    uuid = Column(String, default='')
    file_name = Column(String, default='')
    save_filepath = Column(String, default='')
    file_type = Column(String, default='')
    created_time = Column(Integer, default=0)
    bzid = Column(String, default='')
    file_desc = Column(String, default='')

    class Config:
        orm_mode = True


class YstYhpcsbEntity(Base):

    """
    粤商通_隐患排查上报
    """
    __tablename__ = "yst_yhpcsb"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    bzid = Column(String, default='')
    sfzh = Column(String, default='')
    xm = Column(String, default='')
    qydm = Column(String, default='')
    qymc = Column(String, default='')
    qydz = Column(String, default='')
    qyjb = Column(String, default='')
    fzdc = Column(String, default='')
    fzr = Column(String, default='')
    fzrdh = Column(String, default='')
    gm = Column(String, default='')
    gzcs = Column(String, default='')
    qzjjss = Column(String, default='')
    swxrs = Column(String, default='')
    szdq = Column(String, default='')
    xxdz = Column(String, default='')
    yhdmc = Column(String, default='')
    yhdzt = Column(String, default='')
    ywfzya = Column(String, default='')
    zhlx = Column(String, default='')
    status = Column(Integer, default=0)
    create_time = Column(DateTime, default=datetime.now, comment='数据创建时间')
    update_time = Column(DateTime, default=datetime.now, onupdate=datetime.now, comment='数据更新时间')

    class Config:
        orm_mode = True


class YstQyjcxxEntity(Base):

    """
    粤商通_应急企业基础信息
    """
    __tablename__ = "yst_qyjcxx"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    bzid = Column(String, default='')
    sfzh = Column(String, default='')
    xm = Column(String, default='')
    qydm = Column(String, default='')
    qymc = Column(String, default='')
    qydz = Column(String, default='')
    qyjb = Column(String, default='')
    szdq = Column(String, default='')
    xxdz = Column(String, default='')
    zrrxm = Column(String, default='')
    zrrdh = Column(String, default='')
    status = Column(Integer, default=0)
    create_time = Column(DateTime, default=datetime.now, comment='数据创建时间')
    update_time = Column(DateTime, default=datetime.now, onupdate=datetime.now, comment='数据更新时间')

    class Config:
        orm_mode = True



class YstQyyjyaEntity(Base):

    """
    粤商通_企业应急预案信息
    """
    __tablename__ = "yst_qyyjya"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    bzid = Column(String, default='')
    sfzh = Column(String, default='')
    xm = Column(String, default='')
    qydm = Column(String, default='')
    qymc = Column(String, default='')
    qydz = Column(String, default='')
    qyjb = Column(String, default='')

    bzmd = Column(String, default='')
    bzyj = Column(String, default='')
    qygk = Column(String, default='')
    wxxfx = Column(String, default='')
    syfw = Column(String, default='')
    yjjyyz = Column(String, default='')
    yjczcs = Column(String, default='')
    yjxy = Column(String, default='')
    status = Column(Integer, default=0)
    create_time = Column(DateTime, default=datetime.now, comment='数据创建时间')
    update_time = Column(DateTime, default=datetime.now, onupdate=datetime.now, comment='数据更新时间')

    class Config:
        orm_mode = True



class YstYjylzjEntity(Base):

    """
    粤商通_应急企业基础信息
    """
    __tablename__ = "yst_yjylzj"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    bzid = Column(String, default='')
    sfzh = Column(String, default='')
    xm = Column(String, default='')
    qydm = Column(String, default='')
    qymc = Column(String, default='')
    qydz = Column(String, default='')
    qyjb = Column(String, default='')

    xmmc = Column(String, default='')
    yllx = Column(String, default='')
    zzdw = Column(String, default='')
    
    status = Column(Integer, default=0)
    create_time = Column(DateTime, default=datetime.now, comment='数据创建时间')
    update_time = Column(DateTime, default=datetime.now, onupdate=datetime.now, comment='数据更新时间')

    class Config:
        orm_mode = True

#  待办
class VwYstDaiban(Base):
    __tablename__ = "vw_yst_daiban"

    bzid = Column(String, primary_key=True, index=True)
    sfzh = Column(String)
    sxmc = Column(String)
    xm = Column(String)
    create_time = Column(DateTime)
    update_time = Column(DateTime)
    status = Column(Integer)
    qymc = Column(String)
    qydm = Column(String)

    class Config:
        orm_mode = True

#
#
# 进度表定义
#
#
class YstProcessEntity(Base):
    """
    事项进度
    """
    __tablename__ = "yst_process"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    # 报装ID
    bzid = Column(String, default='')
    # 报装类型
    bzlx = Column(String, default='')
    # 身份证号
    sfzh = Column(String, default='')
    # 申请来源
    sqly = Column(String, default='')
    # 状态
    zt = Column(Integer, default=0)
    # 状态说明
    ztsm = Column(String, default='')
    # 进度说明
    jdsm = Column(String, default='')
    # 登记时间
    djsj = Column(Integer, default=0)
    
    class Config:
        orm_mode = True