#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from fastapi import APIRouter, Request, Depends
from starlette.templating import Jinja2Templates
from fastapi.responses import RedirectResponse
from common.security import valid_access_token,valid_access_token_role
from .login import router as login_router
from . import gateway
from . import dataAnalysis
from . import resourceMonitoring
from . import OneShareProxyHandler
from . import jobs
from . import upload_file
from . import videoResource
from . import Knowledge
from . import emergencyPlans
from . import eventManagement
from . import spatialAnalysis
from . import taskRegistration
from . import riskMonitor
from . import temperaturePrecipitation
from . import pattern
from . import rainfall
from . import infoPublish
from . import riskManagement
from . import onlineRollCall
from . import layerConfiguration
from . import dutyManagement
from . import dataFilling
from . import companyManagement
from . import resourceProvison
from . import yst
from . import comprehensive_search
from . import ThreeProofingResponsible
from . import hazardStandards
from . import city
from extensions import logger
from routers.prod_api import system
from routers.prod_api import auth
from . import qrcode
from . import expert
router = APIRouter()

@router.get("/yzy/callback.html")
async def callback(request: Request, code: str, state: str):
    # 转会vue URL格式
    url="/yjxp/#/yzy/callback?code="+code+"&state="+state
    logger.info("callback: {}", url)
    return RedirectResponse(url)

router.include_router(auth.router, prefix="/auth")
router.include_router(qrcode.router, prefix="/qrcode")
router.include_router(login_router)

router.include_router(system.router, prefix="/system", dependencies=[Depends(valid_access_token_role)])

router.include_router(gateway.router, prefix="/gateway", dependencies=[Depends(valid_access_token_role)])
router.include_router(dataAnalysis.router, prefix="/dataAnalysis", dependencies=[Depends(valid_access_token_role)])
router.include_router(resourceMonitoring.router, prefix="/resource")
router.include_router(jobs.router, prefix="/jobs", dependencies=[Depends(valid_access_token_role)])
router.include_router(OneShareProxyHandler.router, prefix="/oneShare", dependencies=[Depends(valid_access_token_role)])
router.include_router(upload_file.router, prefix="/file")
router.include_router(videoResource.router, prefix="/videoResource", dependencies=[Depends(valid_access_token_role)])
router.include_router(Knowledge.router, prefix="/knowledge", dependencies=[Depends(valid_access_token_role)])
router.include_router(taskRegistration.router, prefix="/taskRegistration", dependencies=[Depends(valid_access_token_role)])
router.include_router(emergencyPlans.router, prefix="/emergency_plan", dependencies=[Depends(valid_access_token_role)])
router.include_router(riskMonitor.router, prefix="/risk_monitor", dependencies=[Depends(valid_access_token_role)])
router.include_router(temperaturePrecipitation.router, prefix="/temperature_precipitation", dependencies=[Depends(valid_access_token_role)])
router.include_router(layerConfiguration.router, prefix="/layerConfiguration", tags=["图层配置"], dependencies=[Depends(valid_access_token_role)])
router.include_router(dataFilling.router, prefix="/dataFilling", dependencies=[Depends(valid_access_token_role)])
router.include_router(resourceProvison.router, prefix="/resource_provison", dependencies=[Depends(valid_access_token_role)])
router.include_router(hazardStandards.router, prefix="/hazardStandards", dependencies=[Depends(valid_access_token_role)])

router.include_router(eventManagement.router, prefix="/event_management", tags=["事件管理"], dependencies=[Depends(valid_access_token_role)])
router.include_router(spatialAnalysis.router, prefix="/spatial_analysis", tags=["空间分析"], dependencies=[Depends(valid_access_token_role)])
router.include_router(pattern.router, prefix="/pattern", tags=["实时测绘"], dependencies=[Depends(valid_access_token_role)])

router.include_router(rainfall.router, prefix="/rainfall", tags=["雨情分析"], dependencies=[Depends(valid_access_token_role)])

router.include_router(infoPublish.router, prefix="/info_publish", dependencies=[Depends(valid_access_token_role)])

router.include_router(riskManagement.router, prefix="/riskManagement", tags=["风险防控"], dependencies=[Depends(valid_access_token_role)])

router.include_router(onlineRollCall.router, prefix="/online_roll_call", tags=["在线点名"], dependencies=[Depends(valid_access_token_role)])

router.include_router(dutyManagement.router, prefix="/duty_management", tags=["值班管理"], dependencies=[Depends(valid_access_token_role)])

router.include_router(companyManagement.router, prefix="/companyManagement", tags=["企业管理"], dependencies=[Depends(valid_access_token_role)]) #企业画像

router.include_router(comprehensive_search.router, prefix="/comprehensive/search", tags=["全局搜索"], dependencies=[Depends(valid_access_token_role)])
router.include_router(ThreeProofingResponsible.router, prefix="/ThreeProofingResponsible", tags=["三防责任人管理"], dependencies=[Depends(valid_access_token_role)])
router.include_router(city.router, prefix="/city", tags=["区划"], dependencies=[Depends(valid_access_token_role)])
router.include_router(expert.router, prefix="/expert", tags=["应急专家"], dependencies=[Depends(valid_access_token_role)])

router.include_router(yst.router, prefix="/yst", tags=["粤商通"]) # 粤商通不需要，单独的验证逻辑
